/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib;

/**
 * History:
 *
 * $Log: BaseConsoleProgram.java,v $
 * Revision 1.5  2007/08/21 07:47:03  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2007/08/06 11:52:22  Trier
 * Änderung Log-System
 *
 * Revision 1.3  2007/07/25 13:02:05  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2007/07/04 11:58:43  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2007/06/20 13:13:41  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2006/12/19 07:02:04  Trier
 * alle Fenster bekommen jetzt das Menu, Swis überarbeitet
 *
 * Revision 1.2  2006/10/16 11:58:14  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1.1.1  2005/01/21 13:57:21  Trier
 * Applikation
 *
 */

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Klasse vereinfacht die Erzeugung von KommandozeilenProgrammen, die eine
 * Connection zum Datenverteiler aufbauen. In der
 * {@link #start(String[] args, boolean exitAfterRun)} wird die
 * Datenverteilerverbindung aufgebaut und &uuml;ber die Singleton-Klasse
 * {@link LogosClientDavConnection} bereitgestellt. Die abstracte Methode run()
 * muss von der konkreten Klasse implementiert werden.
 *
 * Beispiel für eine konkrete Klasse:
 *
 * <pre>
 * public class TestConsoleProgram extends
 * 		de.logosib.dvs.funclib.BaseConsoleProgram {
 *
 * 	protected void run() {
 *       ..;
 *   }
 *
 * 	// Main-Methode
 * 	public static void main(String[] args) {
 * 		(new TestConsoleProgram()).start(args, true);
 * 	}
 * }
 * </pre>
 *
 * @version 1.0
 */
public abstract class BaseConsoleProgram extends java.lang.Object {

	private static de.bsvrz.dua.bastband.bastband.funclib.logger.Logger logger = null;

	/**
	 * Methode zum Starten der Anwendung.
	 *
	 * @param args
	 *            AufrufParameter des Programms.
	 * @param exitAfterRun
	 *            true, wenn nach Beendigung der run-Methode das Programm beendet
	 *            werden soll.
	 */
	public void start(final String[] args, final boolean exitAfterRun) {

		LogosClientDavConnection oClientDavConnection = null;
		try {
			Debug.init(this.getClass().getSimpleName(), new ArgumentList(args));
			System.err.println("sys.funclib.debug.Debug.debugInfo()=" + Debug.debugInfo());
			BaseConsoleProgram.logger = de.bsvrz.dua.bastband.bastband.funclib.logger.LoggerFactory.getLoggerFactory()
					.getLogger();
		} catch (final Exception e) {
			System.err.println("Fehler beim Initialisieren des Loggers.");
			e.printStackTrace();
		}

		try {
			final ClientDavParameters parameters = new ClientDavParameters(args);
			// parameters.setApplicationTypePid("typ.applikationLogos");
			parameters.setApplicationName(this.getClass().getName());
			oClientDavConnection = new LogosClientDavConnection(parameters);
			oClientDavConnection.connect();
			oClientDavConnection.login();
		} catch (final Exception e) {
			System.err.println("Fehler beim Verbinden mit dem Datenverteiler. Programm wird beendet.");
			System.err.println("Ursache:");
			e.printStackTrace();
			System.exit(1);
		}
		try {
			BaseConsoleProgram.addShutdownHook(oClientDavConnection);
			this.run();
		} catch (final Exception e) {
			System.err.println("Fehler zur Laufzeit des Programms.");
			System.err.println("Ursache:");
			e.printStackTrace();
		}
		if (exitAfterRun) {
			System.exit(0);
		}
		final StandardApplicationRunner s;
	}

	/**
	 * wird nach der Erzeugung der Verbindung zum Datenverteiler aufgerufen.
	 *
	 */
	public abstract void run();

	/**
	 * <p>
	 * Erzeugt und registriert einen Shutdown-Thread, der daf&uuml;r sorgt, dass vor
	 * Beendigung der JRE, die angegebene Datenverteilerverbindung getrennt wird.
	 * </p>
	 *
	 * @param davCon
	 *            ClientDavInterface
	 */
	public static void addShutdownHook(final ClientDavInterface davCon) {
		Runtime.getRuntime().addShutdownHook(new ShutDownThread(davCon));
	}

	/**
	 * <p>
	 * Shutdown-Thread, der daf&uuml;r sorgt, dass vor Beendigung der JRE, die beim
	 * Erzeugen des Threads angegebene Datenverteilerverbindung getrennt wird.
	 * </p>
	 */
	public static class ShutDownThread extends Thread {

		ClientDavInterface _ClientDavInterface;

		public ShutDownThread(final ClientDavInterface davCon) {
			_ClientDavInterface = davCon;
		}

		@Override
		public synchronized void run() {
			try {
				if (_ClientDavInterface != null) {
					_ClientDavInterface.disconnect(false, "Anwendung wurde gestoppt");
					if (_ClientDavInterface instanceof LogosClientDavConnection) {
						LogosClientDavConnection.setConnection(null);
					}
					_ClientDavInterface = null;
				}
			} catch (final Exception e) {
				e.printStackTrace();
			}
			System.out.println("Shuting down .. done");
		}
	}

}
