/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.IntegerValueState;

/**
 * History:
 * 2007-07-13 [KS]  Bugfix wegen Zugriffsfehler in Kernsoftware beim Aufruf von
 *                  Data.getScaledValue(String).shortValue() und Skalierung !=1
 *
 * $Log: AtlGeschwindigkeit.java,v $
 * Revision 1.4  2007/08/22 07:48:44  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2007/07/25 13:02:10  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2007/07/13 11:31:07  Strey
 * Bugfix wegen Zugriffsfehler in Kernsoftware beim Zugriff auf Data.getScaledValue().shortValue().
 *
 * Revision 1.1  2007/07/04 11:42:55  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.7  2007/06/14 13:57:47  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.6  2006/11/29 10:48:27  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.5  2006/11/29 09:53:05  Trier
 * getValue liefert jetzt float zurück
 *
 * Revision 1.4  2006/11/13 09:24:12  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/01/25 09:43:11  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/01/21 13:26:16  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1.1.1  2004/03/23 13:14:18  Trier
 * Basis für LogosApps
 *
 * Revision 1.1.1.1  2004/03/23 07:59:52  Trier
 * vrz
 *
 */

/**
 * kapselt die AttributListe atl.geschwindigkeit.
 */
public class AtlGeschwindigkeit {

	private final Data _Data;

	public static final String ATTR_NAME_WERT = "Wert";
	public static final String ATTR_NAME_STATUS = "Status";
	public static final String ATTR_NAME_GUETE = "Güte";

	public static final String ZUSTAND_WERT_NICHT_ERMITTELBAR = "nicht ermittelbar";
	public static final String ZUSTAND_WERT_FEHLERHAFT = "fehlerhaft";
	public static final String ZUSTAND_WERT_NICHT_ERMITTELBAR_FEHLERHAFT = "nicht ermittelbar/fehlerhaft";

	protected boolean _hasState = true;

	public AtlGeschwindigkeit(final Data data) {
		_Data = data;
		_hasState = "atl.geschwindigkeit".equals(data.getAttributeType().getPid());
	}

	public void doInitialize() {
		new AtlStatus(_Data.getItem("Status")).doInitialize();
		new AtlGuete(_Data.getItem("Güte")).doInitialize();
		_Data.getScaledValue("Wert").setText(AtlGeschwindigkeit.ZUSTAND_WERT_NICHT_ERMITTELBAR);
	}

	public IntegerValueRange getRangeOfGeschwindigkeit() {
		return ValueRangeHashtable.getInstance(this._Data.getItem("Wert")).get("att.geschwindigkeit");
	}

	public double getWert() {
		// BUGFIX wegen Exception mit neuer Kernsoftware
		// return _Data.getScaledValue("Wert").shortValue();
		// return (short)(_Data.getScaledValue("Wert").doubleValue());
		return _Data.getScaledValue("Wert").doubleValue();
	}

	public void setWert(final double wert) {
		try {
			if (wert < 0) {
				_Data.getUnscaledValue("Wert").set(wert);
			} else {
				_Data.getScaledValue("Wert").set(wert);
			}
		} catch (final IllegalArgumentException ex) {
			ex.printStackTrace();
			final long lMaxValue = getRangeOfGeschwindigkeit().getMaximum();
			final long lMinValue = getRangeOfGeschwindigkeit().getMinimum();
			throw new IllegalArgumentException("Geschwindigkeitswert konnte nicht auf " + wert
					+ " gesetzt werden. (Min/Max:" + lMinValue + "/" + lMaxValue + ")");
		}
	}

	// public void setWert (int wert) {
	// try {
	// if (wert < 0) {
	// _Data.getUnscaledValue("Wert").set(wert);
	// }
	// else {
	// // _Data.getScaledValue("Wert").set(wert);
	// _Data.getScaledValue("Wert").set( 0.1*wert);
	// }
	// }
	// catch (IllegalArgumentException ex) {
	// ex.printStackTrace();
	// long lMaxValue = getRangeOfGeschwindigkeit().getMaximum();
	// long lMinValue = getRangeOfGeschwindigkeit().getMinimum();
	// throw new
	// IllegalArgumentException("Geschwindigkeitswert konnte nicht auf " +
	// wert + " gesetzt werden. (Min/Max:" +
	// lMinValue+"/"+lMaxValue + ")");
	// }
	// }
	// public void setWert (short wert) {
	// try {
	// if (wert < 0) {
	// _Data.getUnscaledValue("Wert").set(wert);
	// }
	// else {
	// // _Data.getScaledValue("Wert").set(wert);
	// _Data.getScaledValue("Wert").set(wert);
	// }
	// }
	// catch (IllegalArgumentException ex) {
	// long lMaxValue = getRangeOfGeschwindigkeit().getMaximum();
	// long lMinValue = getRangeOfGeschwindigkeit().getMinimum();
	// throw new
	// IllegalArgumentException("Geschwindigkeitswert konnte nicht auf " +
	// wert + " gesetzt werden. (Min/Max:" +
	// lMinValue+"/"+lMaxValue + ")");
	// }
	// }
	public IntegerValueState getZustandWert() {
		return _Data.getScaledValue("Wert").getState();
	}

	public void setZustandWert(final IntegerValueState integerValueState) {
		_Data.getScaledValue("Wert").setState(integerValueState);
	}

	public void setZustandWert(final String zustandsName) {
		_Data.getScaledValue("Wert").setText(zustandsName);
	}

	/**
	 * liefert AtlStatus-Instance zur&uuml;ck, wenn das gekapselte Data vom Typ
	 * atl.geschwindigkeit ist und einen Status besitzt (sonst null).
	 *
	 * @return der ermittelte Status
	 */
	public AtlStatus getStatus() {
		if (_hasState) {
			return new AtlStatus(_Data.getItem("Status"));
		}
		return null;
	}

	/**
	 * liefert true zur&uuml;ck, wenn das gekapselte Data vom Typ
	 * atl.geschwindigkeit ist und einen Status besitzt.
	 *
	 * @return boolean
	 */
	public boolean hasState() {
		return _hasState;
	}

	public AtlGuete getGuete() {
		return new AtlGuete(_Data.getItem("Güte"));
	}

	public boolean isState() {
		return _Data.getUnscaledValue("Wert").isState();
	}

	public boolean isNumber() {
		return !isState();
	}

}
