/**
 * Segment 4 Datenübernahme und Aufbereitung (DUA), SWE 4.10 Ergänzung BASt-Band
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.bastband.gui;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Calendar;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.tree.DefaultMutableTreeNode;

/**
 * Ermoeglicht eine Graphische Darstellung der Parameter der Applikation
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class BastBandGUI extends JDialog implements ActionListener {

	/**
	 * Autogenerated
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * Der Jahr mit dem man im Combobox beginnt
	 */
	final int ANFANG_JAHR = 1995;

	public static final String[] MONATE = { "Januar", "Februar", "März", "April", "Mai", "Juni", "Juli", "August",
			"September", "Oktober", "November", "Dezember" };

	private JPanel hauptPanel;
	/**
	 * Text Feld mit der Zielpfade
	 */
	private JTextField ZielPfad;
	/**
	 * Kombinationsfeld mit dem Anfangjahr
	 */
	private JComboBox<Integer> anfangJahr;
	/**
	 * Kombinationsfeld mit dem Anfangmonat
	 */
	private JComboBox<String> anfangMonat;
	/**
	 * Kombinationsfeld mit dem Endejahr
	 */
	private JComboBox<Integer> endeJahr;
	/**
	 * Kombinationsfeld mit dem Endemonat
	 */
	private JComboBox<String> endeMonat;
	/**
	 * Kontrollkaestchen, ob die Dateien unterschrieben weden sollen
	 */
	private JCheckBox ueberschreiben;

	/**
	 * Taste, die Durchsuchungsdialog öffnet
	 */
	private JButton Durchsuchen;

	/**
	 * Taste, für das Fertigstellen der Operation
	 */
	private JButton BtnFertigstellen;

	/**
	 * Taste, für das Abbrechen der Operation
	 */
	private JButton BtnAbbrechen;

	/**
	 * DerAusgewaehlteFormat
	 */
	private String bastFormat;

	/**
	 * Die RadioButtons zum Formatwahl
	 */
	private ButtonGroup btnGruppeFormat;

	/**
	 * Der baum mit LZZ
	 */
	private ZaehlStellenBaum zaehlStellenBaum;

	/**
	 * Initialsation
	 */
	public BastBandGUI() {
		super();
		initGUI();
	}

	/**
	 * Initialisierung
	 */
	private void initGUI() {
		try {
			// Windows Look & Feel einstellen
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			this.setTitle("Ergänzung BASt-Band");

			hauptPanel = new JPanel();
			this.add(hauptPanel);

			final BoxLayout verticalBox = new BoxLayout(hauptPanel, BoxLayout.Y_AXIS);

			hauptPanel.setLayout(verticalBox);

			final Box box1 = new Box(BoxLayout.X_AXIS);
			hauptPanel.add(box1);

			ZielPfad = new JTextField("");
			ZielPfad.setMaximumSize(new Dimension(ZielPfad.getMaximumSize().width, ZielPfad.getPreferredSize().height));

			Durchsuchen = new JButton("Durchsuchen...");
			Durchsuchen.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(final ActionEvent e) {
					final String pf = BastBandGUI.this.ZielPfad.getText();

					final JFileChooser auswaehleFile = new JFileChooser(new File(pf));

					auswaehleFile.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
					// auswaehleFile.showSaveDialog(new Button());
					final int ret = auswaehleFile.showSaveDialog(BastBandGUI.this.hauptPanel);
					if (ret == JFileChooser.APPROVE_OPTION) {
						BastBandGUI.this.ZielPfad.setText(auswaehleFile.getSelectedFile().getAbsolutePath());
					}
				}
			});

			box1.add(new JLabel("Zielpfad:"));
			box1.add(ZielPfad);
			box1.add(Durchsuchen);

			box1.setBorder(BorderFactory.createTitledBorder("Ziel der BASt-Erstellung"));

			final Box box2 = new Box(BoxLayout.X_AXIS);
			final Box anfang = new Box(BoxLayout.X_AXIS);
			final Box ende = new Box(BoxLayout.X_AXIS);

			box2.add(anfang);
			box2.add(ende);
			hauptPanel.add(box2);

			anfang.setBorder(BorderFactory.createTitledBorder("Anfang (Jahr, Monat)"));
			ende.setBorder(BorderFactory.createTitledBorder("Ende (Jahr, Monat)"));

			final Calendar jetzt = Calendar.getInstance();
			final int jahr = jetzt.get(Calendar.YEAR);
			final Vector<Integer> jahre = new Vector<>();

			for (int i = jahr; i > ANFANG_JAHR; i--) {
				jahre.add(new Integer(i));
			}

			anfangJahr = new JComboBox<>(jahre);
			endeJahr = new JComboBox<>(jahre);
			anfangMonat = new JComboBox<>(BastBandGUI.MONATE);
			endeMonat = new JComboBox<>(BastBandGUI.MONATE);

			anfangJahr.setMaximumSize(
					new Dimension(anfangJahr.getMaximumSize().width, anfangJahr.getPreferredSize().height));

			endeJahr.setMaximumSize(new Dimension(endeJahr.getMaximumSize().width, endeJahr.getPreferredSize().height));

			endeMonat.setMaximumSize(
					new Dimension(endeMonat.getMaximumSize().width, endeMonat.getPreferredSize().height));

			anfangMonat.setMaximumSize(
					new Dimension(anfangMonat.getMaximumSize().width, anfangMonat.getPreferredSize().height));

			anfang.add(anfangJahr);
			anfang.add(anfangMonat);

			ende.add(endeJahr);
			ende.add(endeMonat);

			zaehlStellenBaum = new ZaehlStellenBaum();

			ueberschreiben = new JCheckBox("Existierende Dateien überschreiben");

			final Box box3 = new Box(BoxLayout.Y_AXIS);

			final Box box31 = new Box(BoxLayout.X_AXIS);
			final Box box32 = new Box(BoxLayout.X_AXIS);
			final Box box311 = new Box(BoxLayout.Y_AXIS);
			final Box box312 = new Box(BoxLayout.Y_AXIS);

			box31.add(box311);
			box31.add(box312);

			box31.setBorder(BorderFactory.createTitledBorder("BASt-Bestandsbandformat"));
			box3.add(Box.createHorizontalStrut(10));
			box3.add(box31);
			box3.add(Box.createHorizontalStrut(5));
			box3.add(box32);

			box32.add(ueberschreiben);

			box3.add(Box.createVerticalStrut(10));

			btnGruppeFormat = new ButtonGroup();

			final JRadioButton rBtn1 = new JRadioButton("Version 2004");
			final JRadioButton rBtn2 = new JRadioButton("Sachsen-Anhalt");
			final JRadioButton rBtn3 = new JRadioButton("Mecklenburg-Vorpommern");
			final JRadioButton rBtn4 = new JRadioButton("Test MV");
			final JRadioButton rBtn5 = new JRadioButton("Achslastdaten");

			btnGruppeFormat.add(rBtn1);
			btnGruppeFormat.add(rBtn2);
			btnGruppeFormat.add(rBtn3);
			btnGruppeFormat.add(rBtn4);
			btnGruppeFormat.add(rBtn5);

			rBtn1.addActionListener(this);
			rBtn2.addActionListener(this);
			rBtn3.addActionListener(this);
			rBtn4.addActionListener(this);
			rBtn5.addActionListener(this);

			box311.add(rBtn1);
			box311.add(rBtn2);
			box312.add(rBtn3);
			box312.add(rBtn4);
			box311.add(rBtn5);

			final Box kvs = new Box(BoxLayout.X_AXIS | BoxLayout.Y_AXIS);
			final GridLayout lom = new GridLayout();
			zaehlStellenBaum.setLayout(lom);
			kvs.setBorder(BorderFactory.createTitledBorder("Auswahl Konfigurationsbereiche"));
			kvs.setVisible(true);
			kvs.add(zaehlStellenBaum);

			hauptPanel.add(kvs);

			hauptPanel.add(box3);

			hauptPanel.add(new JSeparator());

			final Box box4 = new Box(BoxLayout.X_AXIS);
			BtnAbbrechen = new JButton("Abbrechen");
			BtnFertigstellen = new JButton("Fertig Stellen");

			box4.add(BtnFertigstellen);
			box4.add(Box.createHorizontalStrut(50));
			box4.add(BtnAbbrechen);

			hauptPanel.add(box4);

			pack();
			this.setSize(500, 750);
			this.setResizable(true);

		} catch (final Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Uebuerpriuft ob die Einstellungen plausibel sind
	 *
	 * @param listLZZ
	 *            Liste der exportierenden LZZ
	 * @return true, wenn die Einstellungen OK sind
	 */
	public boolean ueberprufeEinstellungen(final Collection<String> listLZZ) {
		final File file = new File(ZielPfad.getText());
		if (!file.isDirectory()) {
			JOptionPane.showMessageDialog(this, "Zielpfad ist nicht gültig", "BAStBand", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		final Integer anfangJ = (Integer) this.anfangJahr.getSelectedItem();
		final Integer endeJ = (Integer) this.endeJahr.getSelectedItem();
		final int anfangM = this.anfangMonat.getSelectedIndex();
		final int endeM = this.endeMonat.getSelectedIndex();

		if ((anfangJ > endeJ) || ((anfangJ.equals(endeJ)) && (anfangM > endeM))) {
			JOptionPane.showMessageDialog(this, "Zeitbereich ist negativ", "BAStBand", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		listLZZ.clear();
		zaehlStellenBaum.updateSelections(listLZZ);
		if (listLZZ.isEmpty()) {
			JOptionPane.showMessageDialog(this, "Bitte wählen Sie Langzeitzählstellen aus", "BAStBand",
					JOptionPane.ERROR_MESSAGE);
			return false;
		}

		if (bastFormat == null) {
			JOptionPane.showMessageDialog(this, "Bitte wählen Sie ein Format", "BAStBand", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		return true;
	}

	/**
	 * Ob man die existierende Dateien ueberschreiben soll
	 *
	 * @return true wenn die Dateien ueberschrieben werden sollen
	 */
	public boolean getUeberSchreiben() {
		return !(ueberschreiben.getSelectedObjects() == null);
	}

	/**
	 * Ob man die existierende Dateien ueberschreiben soll
	 *
	 * @param value
	 *            true, wenn die Dateien ueberschrieben werden sollen
	 */
	public void setUeberSchreiben(final boolean value) {
		ueberschreiben.setSelected(value);
	}

	/**
	 * Stellt fest ein neues Hauptknoten in den Zaehlstellenbaum ein
	 *
	 * @param hauptKnoten
	 *            Das neue Hauptknoten
	 */
	public void setZaehlstellenBaumHauptKnoten(final DefaultMutableTreeNode hauptKnoten) {
		zaehlStellenBaum.setHauptKnoten(hauptKnoten);
	}

	/**
	 * Expandiert eine Zeile des Baums
	 *
	 * @param n
	 *            Reihenfolge der Zeile
	 */
	public void expandRow(final int n) {
		zaehlStellenBaum.expandRow(n);
	}

	/**
	 * Stellt fest eine Aktion zum Button Abbreben
	 *
	 * @param al
	 *            die Aktion
	 */
	public void addActionListenerBtnAbbrechen(final ActionListener al) {
		BtnAbbrechen.addActionListener(al);
	}

	/**
	 * Stellt fest eine Aktion zum Button FertigStellen
	 *
	 * @param al
	 *            die Aktion
	 */
	public void addActionListenerBtnFertigStellen(final ActionListener al) {
		BtnFertigstellen.addActionListener(al);
	}

	/**
	 * Stellt fest den AnfangJahr des exporierenen Zeitraums
	 *
	 * @param i
	 *            AnfangJahr
	 */
	public void setAnfangJahr(final Integer i) {
		this.anfangJahr.setSelectedItem(i);
	}

	/**
	 * Stellt fest den AnfangMonat des exporierenen Zeitraums
	 *
	 * @param i
	 *            anfangMonat ( Januar == 0 )
	 */
	public void setAnfangMonat(final int i) {
		this.anfangMonat.setSelectedIndex(i);
	}

	/**
	 * Stellt fest den EndeJahr des exporierenen Zeitraums
	 *
	 * @param i
	 *            EndeJahr
	 */
	public void setEndeJahr(final Integer i) {
		this.endeJahr.setSelectedItem(i);
	}

	/**
	 * Stellt fest den EndeMonat des exporierenen Zeitraums
	 *
	 * @param i
	 *            EndeMonat ( Januar == 0 )
	 */
	public void setEndeMonat(final int i) {
		this.endeMonat.setSelectedIndex(i);
	}

	/**
	 * Stellt fest den Ausgabepfad
	 *
	 * @param pfad
	 *            Pfad zum Verzeichnis
	 */
	public void setAusgabePfad(final String pfad) {
		this.ZielPfad.setText(pfad);
	}

	/**
	 * Erfragt den Pfad zum AusgabeVerzeichnis
	 *
	 * @return der Pfad
	 */
	public String getAusgabePfad() {
		return this.ZielPfad.getText();
	}

	/**
	 * Aktualisiert die Menge der Ausgewaehlten Zaehlstellen
	 *
	 * @param c
	 *            Menge von Zaehlstellen
	 */
	public void updateSelections(final Collection<String> c) {
		zaehlStellenBaum.updateSelections(c);
	}

	/**
	 * Erfragt den AnfangsJahr des exportierenen Zeitraums
	 *
	 * @return AnfangsJahr
	 */
	public Integer getAnfangJahr() {
		return (Integer) this.anfangJahr.getSelectedItem();
	}

	/**
	 * Erfragt den AnfangsMonat des exportierenen Zeitraums
	 *
	 * @return AnfangsMonat ( Januar == 0)
	 */
	public int getAnfangMonat() {
		return this.anfangMonat.getSelectedIndex();
	}

	/**
	 * Erfragt den EndeJahr des exportierenen Zeitraums
	 *
	 * @return EndeJahr
	 */
	public Integer getEndeJahr() {
		return (Integer) this.endeJahr.getSelectedItem();
	}

	/**
	 * Erfragt den EndeMonat des exportierenen Zeitraums
	 *
	 * @return EndeMonat ( Januar == 0)
	 */
	public int getEndeMonat() {
		return this.endeMonat.getSelectedIndex();
	}

	/**
	 * Listener Methode fuer den RadioButtonGroup Wird gerufen, wenn man den
	 * Ausgabeformat aendert
	 */
	@Override
	public void actionPerformed(final ActionEvent e) {
		bastFormat = e.getActionCommand();
		if (bastFormat.equals("Achslastdaten")) {
			this.ueberschreiben.setEnabled(false);
		} else {
			this.ueberschreiben.setEnabled(true);
		}
	}

	/**
	 * Erfragt den ausgewaeuhlten Ausgabeformat
	 *
	 * @return Format des Ausgabefiles
	 */
	public String getBastFormat() {
		return bastFormat;
	}

	/**
	 * Stellt fest den Ausgabeformat in die RadioButtonGruppe ein
	 *
	 * @param bastFormat
	 *            Format des Ausgabefiles
	 */
	public void setBastFormat(final String bastFormat) {
		this.bastFormat = bastFormat;
		final Enumeration<AbstractButton> enumer = btnGruppeFormat.getElements();
		while (enumer.hasMoreElements()) {
			final AbstractButton absBtn = enumer.nextElement();
			final JRadioButton jrb = (JRadioButton) absBtn;
			if (jrb.getActionCommand().equals(bastFormat)) {
				jrb.setSelected(true);
			}
		}
	}
}