/**
 * Segment 4 Datenübernahme und Aufbereitung (DUA), SWE 4.10 Ergänzung BASt-Band
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.bastband.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.plaf.ColorUIResource;
import javax.swing.tree.TreeCellRenderer;

/**
 * Rendert dem Knoten mit KontrollKaestchen
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class KontrollKaestchenKnotenRenderer extends JPanel implements TreeCellRenderer {

	/**
	 * Autogenerated
	 */
	private static final long serialVersionUID = 3461456772121104317L;

	/**
	 * Der Kontrollkaestchen
	 */
	protected JCheckBox check;
	/**
	 * Der Text
	 */
	protected TreeLabel label;

	/**
	 * Standardkonstruktor
	 */
	public KontrollKaestchenKnotenRenderer() {
		setLayout(null);
		add(check = new JCheckBox());
		add(label = new TreeLabel());
		check.setBackground(UIManager.getColor("Tree.textBackground"));
	}

	@Override
	public Component getTreeCellRendererComponent(final JTree tree, final Object value, final boolean isSelected,
			final boolean expanded, final boolean leaf, final int row, final boolean hasFocus) {

		final String stringValue = tree.convertValueToText(value, isSelected, expanded, leaf, row, hasFocus);

		setEnabled(tree.isEnabled());
		check.setSelected(((KontrollKaestchenKnoten) value).isAusgewaehlt());
		label.setFont(tree.getFont());
		label.setText(stringValue);
		label.setIstAusgewaehlt(isSelected);
		label.setFocus(hasFocus);

		// Ikone zum Kontrollkästchen
		if (leaf) {
			label.setIcon(UIManager.getIcon("Tree.leafIcon"));
		} else if (expanded) {
			label.setIcon(UIManager.getIcon("Tree.openIcon"));
		} else {
			label.setIcon(UIManager.getIcon("Tree.closedIcon"));
		}
		return this;
	}

	@Override
	public Dimension getPreferredSize() {
		final Dimension dCheck = check.getPreferredSize();
		final Dimension dLabel = label.getPreferredSize();
		return new Dimension(dCheck.width + dLabel.width,
				(dCheck.height < dLabel.height ? dLabel.height : dCheck.height));
	}

	@Override
	public void doLayout() {

		final Dimension dCheck = check.getPreferredSize();
		final Dimension dLabel = label.getPreferredSize();

		int yCheck = 0;
		int yLabel = 0;

		if (dCheck.height < dLabel.height) {
			yCheck = (dLabel.height - dCheck.height) / 2;
		} else {
			yLabel = (dCheck.height - dLabel.height) / 2;
		}

		check.setLocation(0, yCheck);
		// -2 reprarier ein kleines visuelles Fehler
		check.setBounds(0, yCheck, dCheck.width, dCheck.height - 2);

		label.setLocation(dCheck.width, yLabel);
		label.setBounds(dCheck.width, yLabel, dLabel.width, dLabel.height - 2);
	}

	@Override
	public void setBackground(Color color) {
		if (color instanceof ColorUIResource) {
			color = null;
		}
		super.setBackground(color);
	}

	public TreeLabel getLabel() {
		return label;
	}
}

/**
 * Ein Feld im Baum mit text
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
class TreeLabel extends JLabel {

	/**
	 * Autogenerated
	 */
	private static final long serialVersionUID = 3311296551084690981L;
	/**
	 * Ob der Text ausgewaehlt ist
	 */
	boolean istAusgeweahlt;
	/**
	 * Ob man den Focus hat
	 */
	boolean hatFocus;

	/**
	 * Stellt fest die Farbe des Hintergrunds
	 */
	@Override
	public void setBackground(Color color) {
		if (color instanceof ColorUIResource) {
			color = null;
		}
		super.setBackground(color);
	}

	/**
	 * Malt den text auf den Hintergrund
	 */
	@Override
	public void paint(final Graphics g) {
		String str;
		if ((str = getText()) != null) {
			if (0 < str.length()) {
				if (istAusgeweahlt) {
					g.setColor(UIManager.getColor("Tree.selectionBackground"));
				} else {
					g.setColor(UIManager.getColor("Tree.textBackground"));
				}
				final Dimension d = getPreferredSize();
				int imageOffset = 0;
				final Icon currentI = getIcon();
				if (currentI != null) {
					imageOffset = currentI.getIconWidth() + Math.max(0, getIconTextGap() - 1);
				}
				g.fillRect(imageOffset - 2, 0, d.width - imageOffset, d.height);
				if (hatFocus) {
					g.setColor(UIManager.getColor("Tree.selectionBorderColor"));
					g.drawRect(imageOffset - 2, 0, d.width - 1 - imageOffset, d.height - 4);
				}
			}
		}
		super.paint(g);
	}

	/**
	 * Erfragt die Groesse des Textes
	 */
	@Override
	public Dimension getPreferredSize() {
		Dimension retDimension = super.getPreferredSize();
		if (retDimension != null) {
			retDimension = new Dimension(retDimension.width + 3, retDimension.height + 3);
		}
		return retDimension;
	}

	/**
	 * Stellt fest ob der TextAusgewaehlt ist
	 *
	 * @param istAusgeweahlt
	 *            true, wenn ausgewaehlt
	 */
	void setIstAusgewaehlt(final boolean istAusgeweahlt) {
		this.istAusgeweahlt = istAusgeweahlt;
	}

	/**
	 * Stellt fest den Focus ein
	 *
	 * @param hatFocus
	 *            true, wenn der Focus da liegt
	 */
	void setFocus(final boolean hatFocus) {
		this.hatFocus = hatFocus;
	}
}
